package com.gic.haoban.manage.web.controller;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxApplicationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.api.service.WxApplicationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.AdminRoleVO;
import com.gic.haoban.manage.web.vo.AdminVO;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;

@RestController
public class AdminController extends WebBaseController{
	private static Logger logger  = LoggerFactory.getLogger(AdminController.class);
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
	@Autowired
	private WxApplicationApiService wxApplicationApiService;
	
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;
	
	@RequestMapping("admin-list")
	public HaobanResponse adminList() {
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		WxApplicationDTO application = wxApplicationApiService.selectByWxEnterpriseIdAndApplicationType(wxEnterpriseId, 1);
		WxEnterpriseDTO enterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
		logger.info("【管理员查询】corpid={},siteId={},agentId={}",enterprise.getCorpid(),application.getSiteId(),application.getAgentId());
		String adminList = qywxSuiteApiService.getAdminList(enterprise.getCorpid(), application.getSiteId(), Integer.parseInt(application.getAgentId()));
		logger.info("【管理员查询】userIds={}",JSON.toJSONString(adminList));
		List<String> userIds = new ArrayList<String>();
		if(StringUtils.isNotBlank(adminList)){
			JSONArray jsonArr = JSON.parseArray(adminList);
			for (Object object : jsonArr) {
				JSONObject json = JSON.parseObject(JSON.toJSONString(object));
				userIds.add(json.getString("userid"));
			}
		}
		List<StaffDTO> list = new ArrayList<>();
		List<AdminVO> resultList = new ArrayList<>();
		if(!userIds.isEmpty()){
			list = staffApiService.listByUserIdsAndWxEnterpriseId(userIds,wxEnterpriseId);
			for (StaffDTO staffDTO : list) {
				AdminVO vo = new AdminVO();
				vo.setRoleId(staffDTO.getStaffId());
				vo.setRoleName("管理员");
				List<AdminRoleVO> roleList = new ArrayList<>();
				List<DepartmentDTO> departmentList = staffDTO.getDepartmentList();
				for (DepartmentDTO departmentDTO : departmentList) {
					AdminRoleVO role = new AdminRoleVO();
					role.setDepartmentName(departmentDTO.getDepartmentName());
					role.setUserName(staffDTO.getStaffName());
					role.setUserPhone(staffDTO.getPhoneNumber());
					roleList.add(role);
				}
				vo.setRoleUser(roleList);
				resultList.add(vo);
			}
		}
		return resultResponse(HaoBanErrCode.ERR_1,resultList);
	}

}
