package com.gic.haoban.manage.web.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;

import com.alibaba.fastjson.JSON;
import com.ctrip.framework.apollo.spring.annotation.ApolloConfig;
import com.gic.haoban.common.utils.GlobalVar;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.service.ApplicationSettingApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.WxApplicationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.service.ApplicationApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;

import javax.servlet.http.HttpServletResponse;

@RestController
public class ApplicationController extends WebBaseController{
	private static final Logger log = LogManager.getLogger(ApplicationController.class);
	@Autowired
	private ApplicationApiService applicationApiService;
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService;
	@Autowired
	private WxApplicationApiService wxApplicationApiService;
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;
	@Autowired
	private Config config;
	@Autowired
	private DepartmentApiService departmentApiService;
	@Autowired
	private ApplicationSettingApiService applicationSettingApiService;
	
	@RequestMapping("application-list")
	public HaobanResponse applicationList() {
		List<ApplicationDTO> list = applicationApiService.listApplication();
		return resultResponse(HaoBanErrCode.ERR_1,list);
	}
	@IgnoreLogin
	@RequestMapping("application-open-close-list")
	public HaobanResponse applicationList(String wxEnterpriseId) {
		if(StringUtils.isAnyBlank(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		Map<String,Object> map = new HashMap<>();
		List<ApplicationDTO> list = applicationApiService.listApplication();
		List<String> applicationIds = list.stream().map(s->s.getApplicationId()).collect(Collectors.toList());
		List<ApplicationSettingDTO> result = applicationSettingApiService.listOpenByWxEnterpriseIdAndApplicationIds(wxEnterpriseId,applicationIds);
		//开启
		List<ApplicationDTO> openList = new ArrayList<ApplicationDTO>();
		List<String> openApplicationIds = new ArrayList<>();
		if(CollectionUtil.isNotEmpty(result)){
			openApplicationIds = result.stream().map(s->s.getApplicationId()).collect(Collectors.toList());
			openList = applicationApiService.listByApplications(openApplicationIds);
		}
		//关闭
		List<ApplicationDTO> closeList = new ArrayList<ApplicationDTO>();
		applicationIds.removeAll(openApplicationIds);
		if(CollectionUtil.isNotEmpty(applicationIds)){
			closeList = applicationApiService.listByApplications(applicationIds);
		}		
		map.put("open", openList);
		map.put("close", closeList);
		return resultResponse(HaoBanErrCode.ERR_1,map);
	}
	
	@RequestMapping("open-or-close")
	public HaobanResponse applicationList(String wxEnterpriseId, String applicationId, Integer openFlag) {
		if(StringUtils.isAnyBlank(wxEnterpriseId,applicationId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		if(openFlag == null){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		applicationSettingApiService.saveSetting(wxEnterpriseId, applicationId, openFlag);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	@RequestMapping("has-bind-contract")
	public HaobanResponse hasBindContract(){
		LoginVO login = AuthRequestUtil.getSessionUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		WxApplicationDTO wxApplicationDTO = this.wxApplicationApiService.selectByWxEnterpriseIdAndApplicationType(wxEnterpriseId, 1);
		if(wxApplicationDTO != null){
			return resultResponse(HaoBanErrCode.ERR_1, true);
		}
		return resultResponse(HaoBanErrCode.ERR_1, false);
	}

	@RequestMapping("get-auth-url")
	public HaobanResponse getAuthUrl(){
		String suiteId = config.getSuiteId();
		log.info("应用ID：{}", suiteId);
		String redictUrl = config.getHost() + "callback";
		String authorizationUrl = this.qywxSuiteApiService.getGrantAuthorizationUrl(suiteId, redictUrl);
		return resultResponse(HaoBanErrCode.ERR_1, authorizationUrl);
	}

	@RequestMapping("callback")
	public void callback(String auth_code, HttpServletResponse response){
		String suiteId = config.getSuiteId();
		String result = this.qywxSuiteApiService.corpAuth(suiteId, auth_code);
		log.info("企业微信回调结果：{}. {}", auth_code, result);
		QywxCorpInfoDTO dto = JSON.parseObject(result, QywxCorpInfoDTO.class);
		WxApplicationDTO applicationDTO = new WxApplicationDTO();
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getEnterpriseBycorpId(dto.getCorpid());
		if(enterpriseDTO != null){
			applicationDTO.setName(dto.getAgentName());
			applicationDTO.setWxEnterpriseId(enterpriseDTO.getWxEnterpriseId());
			applicationDTO.setAgentId(dto.getAgentid());
			applicationDTO.setApplicationType(1);
			applicationDTO.setAgentName(dto.getAgentName());
			applicationDTO.setSiteId(dto.getSuiteId());
			wxApplicationApiService.addSuite(applicationDTO);
			this.departmentApiService.initwxDepartment(dto.getCorpid(), suiteId, enterpriseDTO.getWxEnterpriseId());
			enterpriseDTO.setContactFlag(1);
			wxEnterpriseApiService.update(enterpriseDTO);
		}
		String redirectUri = config.getHost() + "index";
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		try {
			response.sendRedirect(redirectUri);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@RequestMapping("cancal-suite")
	@IgnoreLogin
	public HaobanResponse cancelSuite(String corpId, String suiteId){
		this.wxApplicationApiService.cancelSuite(corpId, suiteId);
		return resultResponse(HaoBanErrCode.ERR_0, "s");
	}

	@Autowired
	private QywxUserApiService qywxUserApiService;
	@RequestMapping("list-external-user")
	@IgnoreLogin
	public Object list(String corpid, String suiteid, String userid){
		return this.qywxUserApiService.listExternalUserid(corpid, suiteid, userid);
	}

	@RequestMapping("corp-external-user")
	@IgnoreLogin
	public Object corpuserid(String corpid, String suiteid){
		return this.qywxUserApiService.listCorpExternalUser(corpid, suiteid);
	}

	@RequestMapping("get-external-user-info")
	@IgnoreLogin
	public Object userInfo(String corpid, String suiteid, String userid){
		return resultResponse(HaoBanErrCode.ERR_0, this.qywxUserApiService.getExternalUseridInfo(corpid, suiteid, userid));
	}
}
