package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.AppStaffVo;
import com.gic.haoban.manage.web.vo.InfoVo;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

@RestController
public class InfoController extends WebBaseController{
//	public static final  String siteId = "ww2c34dc56739bb105";

	@Autowired
	private QywxCorpApiService qywxCorpApiService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;

    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;

    @Autowired
    private QywxUserApiService qywxUserApiService;

    private static Logger logger= LoggerFactory.getLogger(InfoController.class);

	@Autowired
	private Config config;

	@RequestMapping("get-info-by-code")
    public HaobanResponse getInfoByCode(String code) {
        String userDTOJson = qywxCorpApiService.getCodeSession(config.getWxSuiteid(),code);
        logger.info("userDTOJson=============>{}",userDTOJson);
        UserDTO user = JSON.parseObject(userDTOJson, UserDTO.class);
        if(user == null) {

            return resultResponse(HaoBanErrCode.ERR_6);
        }
        String userId = user.getUserid();
        String corpId = user.getCorpid();
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getEnterpriseBycorpId(corpId);
        StaffDTO loginStaff = null;
        if(enterprise != null){
            loginStaff = staffApiService.selectByUserIdAndEnterpriseId(userId,enterprise.getWxEnterpriseId());
        }
        if(loginStaff == null){
            if(enterprise != null){
                staffApiService.wxGetAdd(userId,enterprise.getWxEnterpriseId());
                loginStaff = staffApiService.selectByUserIdAndEnterpriseId(userId,enterprise.getWxEnterpriseId());
            }
        }
        if(loginStaff == null){
            return resultResponse(HaoBanErrCode.ERR_600001);
        }

        String wxEnterpriseId = loginStaff.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
        String accesstoken = qywxCorpApiService.getCorpAccessToken(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid());
        InfoVo infoVo = new InfoVo();
        infoVo.setAccessToken(accesstoken);
        infoVo.setStaffId(loginStaff.getStaffId());
        infoVo.setWxEnterpriseId(wxEnterpriseId);
        infoVo.setStaffName(loginStaff.getStaffName());
        infoVo.setPhoneNumber(loginStaff.getPhoneNumber());
        infoVo.setNationcode(loginStaff.getNationCode());
        return resultResponse(HaoBanErrCode.ERR_1,infoVo);

    }

    @RequestMapping("get-user-by-member-code")
    public HaobanResponse getInfoByMemberCode(String code, String gicEnterpriseId) {
        EnterpriseDetailDTO enterpriseDetailDTO = wxEnterpriseRelatedApiService.getByEnterpriseId(gicEnterpriseId);
        if (enterpriseDetailDTO == null) {
            logger.info("改企业没有关系好办:{}", gicEnterpriseId);
            return resultResponse(HaoBanErrCode.ERR_500001);
        }

        WxEnterpriseDTO enterpriseDTO = wxEnterpriseApiService.getOne(enterpriseDetailDTO.getWxEnterpriseId());
        if (enterpriseDetailDTO == null) {
            logger.info("改企业不存在:{}", gicEnterpriseId);
            return resultResponse(HaoBanErrCode.ERR_400002);
        }
        String userDTOJson = qywxUserApiService.getUserByCode(enterpriseDTO.getCorpid(), enterpriseDTO.getMemberSecret(), code);
        logger.info("会员微信登录信息=============>{}", userDTOJson);
        UserDTO user = JSON.parseObject(userDTOJson, UserDTO.class);
        if (user == null) {

            return resultResponse(HaoBanErrCode.ERR_6);
        }
        String userId = user.getUserid();
        String corpId = user.getCorpid();

        Map<String, String> ret = new HashMap<>();
        ret.put("userId", userId);
        ret.put("corpId", corpId);

        return resultResponse(HaoBanErrCode.ERR_1, ret);

    }
	
	@RequestMapping("save-user-login-log")
	public HaobanResponse saveUserLoginLog(UserLoginLogDTO userLoginLogDTO) {
		staffApiService.insertUserLoginLog(userLoginLogDTO);
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}
	
	@RequestMapping("get-staff-info")
	public HaobanResponse getStaffInfo(String phoneNumber) {
        if (StringUtils.isBlank(phoneNumber)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        List<StaffDTO> list = staffApiService.listByPhoneNumber(phoneNumber);
        if(CollectionUtil.isNotEmpty(list)){
        	List<AppStaffVo> staffs = EntityUtil.changeEntityListByJSON(AppStaffVo.class, list);
        	Set<String> wxEnterpriseIds = list.stream().map(s->s.getWxEnterpriseId()).collect(Collectors.toSet());
        	List<YwWxEnterpriseDTO> enterprises = wxEnterpriseApiService.listByIds(wxEnterpriseIds);
        	Map<String,YwWxEnterpriseDTO> map = com.gic.commons.util.CollectionUtil.toMap(enterprises, "wxEnterpriseId");
        	for(AppStaffVo vo:staffs){
        		String wxEnterpriseId = vo.getWxEnterpriseId();
        		String corpName = map.get(wxEnterpriseId) == null?"":map.get(wxEnterpriseId).getCorpName();
        		vo.setWxEnterpriseName(corpName);
        	}
    		return resultResponse(HaoBanErrCode.ERR_1,staffs);
        }
		return resultResponse(HaoBanErrCode.ERR_1,new ArrayList<>());
		
	}
}
